@@ -1,5 +1,19 @@
 context("datasets")
 
+test_that("extra arguments passed onto request", {
+  skip_if_no_auth()
+
+  insert_dataset("bigrquery-examples", "test_temp",
+    friendlyName = "xxx",
+    description = "yyy"
+  )
+  on.exit(delete_dataset("bigrquery-examples", "test_temp"))
+
+  ds <- get_dataset("bigrquery-examples", "test_temp")
+  expect_equal(ds$friendlyName, "xxx")
+  expect_equal(ds$description, "yyy")
+})
+
 test_that("list_datasets validates correctly", {
   expect_error(list_datasets(1),
                ".*project is not a string.*")
